;A16 BIOS v0.1
;(C) 2020 AiO Softworks written by Joseph Rettler
;This ROM file contains a basic boot routine, common error messages, and typical system functions
;Directives are designed for the A16 Assembler v0.67 and later!
;Instruction set is designed for the A16 Emulator v0.76c and later!

;Define character constants for greater readability
.c cr=$000D
.c _=$0020
.c /=$002F
.c 0=$0030
.c equ=$003D

;Define valid RAM range
.c ramStart=$0000
.c ramEnd=$0100

;Limit file size to 32K words for now, the final version will be smaller
;We are going to utilize the upper half of address space for our ROM
.org $8000
.start
init:				;Reset entry point
	nop			;First word is a nop
	psh			;Misc push until emulator is fixed
	sid			;Disable interrupts during POST
;Initialize machine
;rsp			;Reset stack pointer
;Opening messages
	pshn /bootMessageL3	;Push addresses of boot messages onto stack
	pshn /bootMessageL2
	pshn /bootMessageL1
	jsr /print		;Print boot messages in order with CRs between
	ldan \cr
	snda 0
	jsr /print
	ldan \cr
	snda 0
	jsr /print
	ldan \cr
	snda 0
;Beginning the POST
;Step 1:	RAM test
	ldan $A5A5		;Set the test value
	ldxn \ramStart		;Set the start address
	ldyn \ramEnd		;Set the end address
	jsr /testRAM
;Skip the second RAM test for speed
;txa
;jsph /decPrint			;Tell the user how many valid bytes we have
	pshn /256			;Fake it 'till you make it
	jsr /print
	pshn /messageRAMAvailable	;Print the memory message
	jsr /print
	ldan \cr
	snda 0
;Step 2:	
;POST is done
	pshn /messageReady	;Print the Ready prompt
	jsr /print
	cid			;Reenable interrupts after POST is done
	hlt			;Halt for now

;Configurable RAM test loop
;Parameters:	A = Test value
;		X = Start address
;		Y = End address
;Overwrites:	B,X
;Returns:	X = Size of valid RAM
testRAM:
	tab			;Copy value to B
_testRAMLoop:
	staax 0			;Store test byte at 0+X
	ldaax 0			;Load byte from 0+x
	cpb			;Make sure we loaded the right data
	bne /_RAMTestError	;We didn't get the right data back
	cpxy			;Check if we're at the end of the range
	bge /_RAMTestDone	;We are done
	inx			;Skip every other byte to shorten execution time
	inx			;Move to the next byte
	jmp /_testRAMLoop	;Continue looping until we are done
_RAMTestDone:
	rts			;We are done
_RAMTestError:
	pshx			;Preserve X
	pshn /errorRAMFault	;Push addresses of error messages onto stack
	pshn /errorMessage
	ldan \cr
	snda 0
	jsr /print
	jsr /print
	pop			;Get X into A
	jsph /decPrint		;Print X as decimal
	ldan \cr
	snda 0
	pshn /systemHalted	;Inform the user we have halted
	jsr /print
	hlt			;Halt the system
;Print zero-terminated ASCII string
;Parameters:	POP = Address of string to print
;Overwrites:	A,X
;Returns:	None
print:
	popx			;Pull address from stack
_printLoop:
	ldaax 0			;Load next byte. Adding X to 0 results in the address, X
	bzs /_donePrinting	;If we just loaded a 0, exit the loop
	snda 0			;Send the byte we loaded to the terminal
	inx			;Move to the next position
	jmp /_printLoop		;Loop until we're done
_donePrinting:
	rts
;Print decimal representation of register
;Call with a JSPH only!
;Parameters:	A = Input value
;Overwrites:	Y
;Returns:	None
decPrint:
	ldbn 10			;Cheeky falsity for ease of use
	jsph /basePrint
	rspop

;Print base n representation of register
;Call with a JSPH ONLY!
;Parameters:	A = Input value
;		B = Desired base
;Overwrites:
;Returns:	None
basePrint:
_basePrintLoop:
_basePrintDone:
	rspop			;Haven't written this yet

;SLOW Integer Multiplication of two values using the addition method
;Parameters:	A = LHS input
;		B = RHS input
;Overwrites:	A,B,X,Y
;Returns:	A = Result
slowMul:
	tby			;Put B into Y for counter comparison
	ldxn 0			;Initialize counter
	tab			;Copy A into B for addition
	ldan 0			;Clear Accumulator
_slowMulLoop:
	cpxy
	beq /_doneSlowMultiplying	;Stop looping if X=Y
	add			;Add values and increment X
	inx
	jmp /_slowMulLoop	;Keep looping until we're done
_doneSlowMultiplying:
	rts
;SLOW Integer Exponent calculation
;Parameters:	A = Number
;		B = Exponent
;Overwrites:	
;Returns:	
slowExponent:
	tby
	ldxn 0
_slowExpLoop:
	jmp /_slowExpLoop
_doneSlowExponent:
	rts
;SLOW Integer Division of two values using the subtraction method
;Parameters:	A = Numerator
;		B = Denominator
;Overwrites:	A,X
;Returns:	A = Remainder
;		X = Quotient
slowDiv:
	ldxn 0			;Initialize counter
_slowDivLoop:
	cpb
	blt /_doneSlowDividing	;Done if B>A
	sub			;Subtract B from A and increment X if not
	inx
	jmp /_slowDivLoop	;Loop until we're done
_doneSlowDividing:
;txa			;Move result into Accumulator and return
	rts
;FAST Integer Division of two values using the shifting method
;Parameters:	A = Numerator
;		B = Denominator
;Overwrites:	X,Y
;Returns:
fastDiv:
	ldyn 15			;Maximum of 16 shifts
_fastDivLoop:
	cpxy			;Check if we are at the end of our shifts
	beq /_doneFastDividing
	inx
_doneFastDividing:
	rts			;We are done

;Interrupt 2 handler
;Priority controller is on INT2 and I/O 5
;Overwrites:	A
int2:
	psh			;Preserve Accumulator
	rcva 5			;Get interrupt ID
	pop			;Return Accumulator
	rti			;Exit

;System messages
.org $F000
bootMessageL1:				;Boot messages
.z "A16 BIOS v0.1"
bootMessageL2:
.z "(C)2020 AiO Softworks"
bootMessageL3:
.z "RAM test"
messageDone:				;A task is done
.z "Done."
messageReady:				;System Ready prompt
.z "Ready."
.rel -$1
.d $000D
.d $0000
messageRAMAvailable:			;How much memory is available
.z " Bytes available"

;Error messages
errorMessage:				;Opening line for all errors
.z "An error occurred:"
systemHalted:				;System halted
.z "System halted."
errorRAMFault:				;RAM error detected, show the address
.z "Faulty RAM at "
errorInitStep:				;Other error during initialization
.z "Initialization fault in step "
errorDisplay:				;Display controller failure
.z "Display fault"
errorDisk:				;Disk controller fault reported
.z "Disk controller fault number "

256:
.z "256"

;CPU Internal bytes
;All vectors cause a reset right now
.org $FFFA		;Software interrupt vector
.d $8000
.org $FFFB		;Interrupt 1 Vector
.d $8000
.org $FFFC		;Interrupt 2 Vector
.d $8000
.org $FFFD		;Stack configuration value
.d $41FF
.org $FFFE		;Stack pointer initial value
.d $0100
.org $FFFF		;Reset vector
.d $8000
.rel -$1
.end
